/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.collections.fields;

import java.io.IOException;

import junit.framework.Assert;

import org.builtu.collections.fields.Fields;
import org.builtu.testing.BasicTestCase;
import org.builtu.util.commutil.Util;
import org.junit.Test;


public class TestFields extends BasicTestCase {
    @Test
    public void testCreate() {
        new Fields(new String[] {"aaa"});
    }

    @Test public void testOne() {
        final Fields f = new Fields(new String[] { "field" });
        assertEquals(1, f.length);
        assertEquals(null, f.get(0));
        assertTrue(f.containsName("field"));
        assertTrue(!f.containsName("someotherfield"));
        assertEquals("field", f.name(0));
        f.set(0, "aaa");
        assertEquals("aaa", f.get(0));
        f.clear();
        assertEquals(null, f.get(0));
    }
    
    @Test public void testNoField_get() {
        final Fields f = new Fields(new String[] { "field" });
        try {
            f.get("non-existing field");
            Assert.fail("Should throw IndexOutOfBoundsException when no field"+
                    "found");
        } catch (final IndexOutOfBoundsException e) {}
    }
    @Test public void testNoField_set() {
        final Fields f = new Fields(new String[] { "field" });
        try {
            f.set("non-existing field", "");
            Assert.fail("Should throw IndexOutOfBoundsException when no field"+
                    "found");
        } catch (final IndexOutOfBoundsException e) {}
    }
    
    @Test public void testSerializable() 
    	throws IllegalArgumentException, IOException, ClassNotFoundException 
	{
    	final Fields orig = new Fields(new String[] { "field" });
    	orig.set(0, "value");
    	final Fields restored = (Fields) assertSerializable(orig);
    	Assert.assertEquals("value", restored.get(0));
    	restored.set(0, "value2");
    	Assert.assertEquals("value", orig.get(0));
	}    
    
    @Test public void testEquals() {
    	final Fields one = rndFields(), another = rndFields();
    	assertEquals(one, one);
    	assertFalse(one.equals(another));
    	final String [] names = new String [one.length];
    	for (int i=0; i<one.length; i++) names[i] = one.name(i);
    	final Fields copy = new Fields(names);
    	for (int i=0; i<one.length; i++) copy.set(one.name(i), one.get(i));
    	assertEquals(one, copy);
    }
    
    @Test public void testSerialize() throws IOException, ClassNotFoundException {
    	final Fields fields = new Fields(new String [] {"field"});
    	fields.set("field", "value");
    	final Fields restored = (Fields) assertSerializable(fields);
    	Assert.assertEquals("value", restored.get("field"));
    }
    
    private Fields rndFields() {
    	final int N = rnd(20)+1;
    	final String [] names = new String [N];
    	for (int i=0; i<N; i++) {
    		String name = rndString().intern();
    		while (Util.indexOf(names, name)>=0) name = rndString().intern();
    		names[i] = name;
    	}
    	final Fields res = new Fields(names);
    	for (int i=0; i<N; i++) res.set(names[i], rndString());
    	return res;
    }
}

