/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.collections.graph;

import junit.framework.Assert;

import org.builtu.collections.graph.Node;
import org.builtu.collections.graph.NodeImpl;
import org.builtu.collections.graph.NodeVisitor;
import org.builtu.collections.graph.TreeUtil;
import org.builtu.testing.BasicTestCase;
import org.junit.Test;


public class TestNodeRandomCount extends BasicTestCase {

	@Test public void testCount() {
		final int N = 1+rnd(1000);
		final Node<Integer> root = buildRandomTree(N);
		Assert.assertEquals(N, TreeUtil.count(root));
	}
	
	private static class CountingVisitor implements NodeVisitor<Integer> {

		public int sum = 0;
		
		@Override public void visit(final Node<Integer> node) {
			sum += node.getValue();
		}
		
	}
	
	@Test public void testSum() {
		final int N = 1+rnd(1000);
		final Node<Integer> root = buildRandomTree(N);
		final CountingVisitor visitor = new CountingVisitor();
		TreeUtil.forAll(root, visitor);
		final int k = N+1;
		Assert.assertEquals((k*k-k)/2, visitor.sum);
	}

	private NodeImpl<Integer> buildRandomTree(int N) {
		assert N>0;
		final NodeImpl<Integer> res = new NodeImpl<Integer>(String.valueOf(N));
		res.setValue(N);
		N--;
		while (N>0) {
			N -= genTree(res, N);
		}
		return res;
	}

	private int genTree(final NodeImpl<Integer> res, int n) {
		int children = rnd(n+1);
		int generated = 0;
		while (children>0) {
			final NodeImpl<Integer> child = new NodeImpl<Integer>(String.valueOf(n));
			child.setValue(n);
			res.addChild(child);
			n--;
			children--;
			generated++;
			final int generatedNow = genTree(child, n);
			n -= generatedNow;
			generated += generatedNow;
			children -= generatedNow; 
			assert n>=0;
		}
		return generated;
	}
	
	
}
