/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.config.module;

import java.text.MessageFormat;
import java.util.List;

import org.builtu.util.commutil.StringUtil;


public class ConfigParameterNotFoundError extends RuntimeException {
	
	private final String key;
	private final String [] configFiles;
	
	public ConfigParameterNotFoundError(final String [] configFiles, final String key) {
		this.key = key;
		this.configFiles = configFiles;
	}
	
	public String getKey() {
		return key;
	}
	
	public String[] getConfigFiles() {
		return configFiles;
	}

	
	private static final String MESSAGE = "\"{0}\" not found. " +
									      "Please check these files (first file overrides next): {1} " +
									      "and add {2}YOUR_VALUE_HERE{3}";
	
	@Override public String getMessage() {
		final List<String> keyParts = StringUtil.split_list(key, '.');
		final Object [] args = {
				key, buildConfigs(), buildOpenTags(keyParts), buildCloseTags(keyParts)
		};
		return MessageFormat.format(MESSAGE, args);
	}

	private String buildConfigs() {
		return StringUtil.join(", ", configFiles);
	}

	private String buildOpenTags(final List<String> keyParts) {
		final StringBuilder b = new StringBuilder(key.length()+keyParts.size()<<1);
		for (final String part : keyParts) {
			b.append('<').append(part).append('>');
		}
		return b.toString();
	}

	private Object buildCloseTags(final List<String> keyParts) {
		final StringBuilder b = new StringBuilder(key.length()+keyParts.size()*3);
		for (int i=keyParts.size()-1; i>=0; i--) {
			b.append("</").append(keyParts.get(i)).append('>');
		}
		return b.toString();
	}

	private static final long serialVersionUID = 1L;

}
