/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.util.clown;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

class ComplexClassExternalizable implements Externalizable {

	private List<ComplexClassExternalizable> children;
	private int fInt;
	
	public ComplexClassExternalizable() {}
	
	public ComplexClassExternalizable(final Random rnd) {
		this(rnd, 0);
	}
	
	public ComplexClassExternalizable(final Random rnd, final int level) {
		fInt = rnd.nextInt();
		if (level < 10) {
			children = new ArrayList<ComplexClassExternalizable>();
			final int N = rnd.nextInt(level+1);
			for (int i=0; i<N; i++) {
				children.add(new ComplexClassExternalizable(rnd, level+1));
			}
		} else {
			children = null;
		}
	}

	private static final long serialVersionUID = 1L;

	@SuppressWarnings("unchecked")
	@Override public void readExternal(final ObjectInput in) 
		throws IOException, ClassNotFoundException 
	{
		fInt = in.readInt();
		children = (List<ComplexClassExternalizable>) in.readObject();
	}

	@Override public void writeExternal(final ObjectOutput out) throws IOException {
		out.writeInt(fInt);
		out.writeObject(children);
	}

	@Override public boolean equals(Object obj) {
		if (obj == this) return false; // This is violation, but we need to check this is really is a deep copy
		if (!(obj instanceof ComplexClassExternalizable)) return false;
		final ComplexClassExternalizable other = (ComplexClassExternalizable) obj;

		return this.fInt == other.fInt && ComplexClassBase.arraysEqual(this.children, other.children);
	}
	
	

}
