/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.util.commutil;

import java.lang.ref.WeakReference;

/**
 * Weak reference that supports {@link Ref#equals(Object)} and 
 * {@link Ref#hashCode()} methods
 * @author borisman
 */
public class Ref<T> extends WeakReference<T>
{
  private final int hashCode;
  
  public Ref(final T referent) {
    super(referent);
    hashCode = referent.hashCode();
  }

  /**
   * Always returns hashCode of the underlying object
   */
  @Override public int hashCode()
  {
    return hashCode;
  }

  /**
   * Returns true only when another object is Ref<T> and either both references
   * have lost the internal object or these objects are equal 
   */
  @Override public boolean equals(final Object obj)
  {
    if (obj == this) return true;
    if (!(obj instanceof Ref<?>)) return false;
    final Ref<?> rhs = (Ref<?>) obj;
    final T ref = super.get();
    final Object otherRef = rhs.get();
    if (ref == null || otherRef == null) 
      return false;
    else
      return ref.equals(otherRef);
  }

}
