/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.util.commutil;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.UnsupportedEncodingException;
import java.lang.ref.WeakReference;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;

import org.builtu.exceptions.FatalError;
import org.builtu.exceptions.StringFormatException;


public class Util {
    
	public static <E> int indexOf(final E[] coll, final E elem) {
        if(elem == null) return -1;
        for (int i = 0; i < coll.length; i++)
            if (elem.equals(coll[i]))
                return i;
        return -1;
    }

    public static int indexOf(final byte[] coll, final byte elem) {
        for (int i = 0; i < coll.length; i++) if (elem == coll[i]) return i;
        return -1;
    }
    
    public static int indexOf(final int[] coll, final int elem) {
        for (int i = 0; i < coll.length; i++) if (elem == coll[i]) return i;
        return -1;
    }

    public static int indexOf(final long [] coll, final long elem) {
        for (int i = 0; i < coll.length; i++) if (elem == coll[i]) return i;
        return -1;
    }

	public static <E> int indexOfExact(final E[] coll, final E elem) {
        if(elem == null) return -1;
        for (int i = 0; i < coll.length; i++)
            if (elem == coll[i])
                return i;
        return -1;
    }
	
    public static int indexOf(final char[] coll, final char elem) {
        for (int i = 0; i < coll.length; i++)
            if (elem == coll[i])
                return i;
        return -1;
    }

	public static byte[] serialize(final Object obj) {
		final ByteArrayOutputStream st = new ByteArrayOutputStream();
		final ObjectOutputStream os;
		try {
			os = new ObjectOutputStream(st);
			os.writeObject(obj);
			os.close();
		} catch (final IOException e) {
			throw new FatalError(e);
		}
		return st.toByteArray();
	}

	public static Object deserialize(final byte [] data) throws ClassNotFoundException {
		final ByteArrayInputStream st = new ByteArrayInputStream(data);
		final ObjectInputStream is;
		try {
			is = new ObjectInputStream(st);
			final Object res = is.readObject();
			is.close();
			return res;
		} catch (final IOException e) {
			throw new FatalError(e);
		}
	}

	public static<K1, V> V get(final Map<K1, V> map, final K1 key) {
		return map.get(key);
	}

	public static<K1, K2, V> V get(final Map<K1, Map<K2, V>> map, final K1 key1, final K2 key2) {
		final Map<K2, V> map2 = get(map, key1);
		if (map2 == null) 
			return null;
		else
			return map2.get(key2);
	}

	public static<K1, K2, K3, V> V get(
			final Map<K1, Map<K2, Map<K3, V>>> map, 
			final K1 key1, 
			final K2 key2,
			final K3 key3
	) {
		final Map<K3, V> map2 = get(map, key1, key2);
		if (map2 == null) 
			return null;
		else
			return map2.get(key3);
	}

	public static<K1, K2, K3, K4, V> V get(
			final Map<K1, Map<K2, Map<K3, Map<K4, V>>>> map, 
			final K1 key1, 
			final K2 key2,
			final K3 key3,
			final K4 key4
	) {
		final Map<K4, V> map2 = get(map, key1, key2, key3);
		if (map2 == null) 
			return null;
		else
			return map2.get(key4);
	}
	
	public static<K1, V> V put(final Map<K1, V> map, final K1 key, final V value) {
		return map.put(key, value);
	}

	public static<K1, K2, V> V put(final Map<K1, Map<K2, V>> map, final K1 key1, final K2 key2, final V value) {
		Map<K2, V> submap = map.get(key1);
		if (submap == null) {
			submap = new HashMap<K2, V>();
			map.put(key1, submap);
		}
		return submap.put(key2, value);
	}

	public static<K1, K2, K3, V> V put(
			final Map<K1, Map<K2, Map<K3, V>>> map, 
			final K1 key1, 
			final K2 key2, 
			final K3 key3,
			final V value
	) {
		Map<K2,  Map<K3, V>> submap = map.get(key1);
		if (submap == null) {
			submap = new HashMap<K2,  Map<K3, V>>();
			map.put(key1, submap);
		}
		return put(submap, key2, key3, value);
	}
	
	public static<K1, K2, K3, K4, V> V put(
			final Map<K1, Map<K2, Map<K3, Map<K4, V>>>> map, 
			final K1 key1, 
			final K2 key2, 
			final K3 key3,
			final K4 key4,
			final V value
	) {
		Map<K2, Map<K3, Map<K4, V>>> submap = map.get(key1);
		if (submap == null) {
			submap = new HashMap<K2, Map<K3, Map<K4, V>>>();
			map.put(key1, submap);
		}
		return put(submap, key2, key3, key4, value);
	}

	public static<K1, K2, K3, K4, K5, V> V put(
			final Map<K1, Map<K2, Map<K3, Map<K4, Map<K5, V>>>>> map, 
			final K1 key1, 
			final K2 key2, 
			final K3 key3,
			final K4 key4,
			final K5 key5,
			final V value
	) {
		Map<K2, Map<K3, Map<K4, Map<K5, V>>>> submap = map.get(key1);
		if (submap == null) {
			submap = new HashMap<K2, Map<K3, Map<K4, Map<K5, V>>>>();
			map.put(key1, submap);
		}
		return put(submap, key2, key3, key4, key5, value);
	}
	
    public static <E> List<List<E>> splitList(final List<E> source,
            final int parts) {
        if (parts <= 0)
            throw new IllegalArgumentException(parts + "<=0");
        final List<List<E>> res = new ArrayList<List<E>>(parts);
        final int len = source.size();
        final int perList = len / parts + (len % parts == 0 ? 0 : 1);
        for (int i = 0; i < parts; i++) {
            final List<E> part = new ArrayList<E>();
            res.add(part);
            final int from = i * perList;
            if (from >= len)
                continue;
            int to = (i + 1) * perList;
            if (to > len)
                to = len;
            part.addAll(source.subList(from, to));
        }
        assert splitList_countOfElements(res) == len;
        return res;
    }

    private static <E> int splitList_countOfElements(final List<List<E>> list) {
        int res = 0;
        for (final List<E> sublist : list) {
            res += sublist.size();
        }
        return res;
    }
    
    public static StringBuilder loadUTF(final InputStream in) throws IOException {
    	final BufferedReader r = new BufferedReader(new InputStreamReader(in, StringUtil.utf8Charset));
    	final StringBuilder res = new StringBuilder(4096);
    	String line = r.readLine();
    	while (line != null) {
    		res.append(line);
    		res.append('\n');
    	}
    	return res;
    }

    public static String extensionFromPath(final String pathname) {
        final int idx = pathname.lastIndexOf('.');
        if (idx < 0)
            return null;
        else
            return pathname.substring(idx + 1).intern();
    }

    /**
     * Escape special characters (CR, LF, Tab, FF) in string according to
     * regular Java (and C) string escaping scheme. This function also encodes
     * quotes (just in case). For example, if you escape <code>ThisCRisLFaTAB\string"'</code>
     * you will get <code>This\ris\na\t\\string\"\'</code>
     *
     *
     * @param str
     *            to be escaped
     * @see Util#unescape(String) for converting back
     * @return escapen string
     */
    public static String escape(final String str) {
        final StringBuilder res = new StringBuilder(str.length() << 1);
        for (int i = 0; i < str.length(); i++) {
            final char c = str.charAt(i);
            switch (c) {
            case '\'':
                res.append("\\\'");
                break;
            case '\"':
                res.append("\\\"");
                break;
            case '\\':
                res.append("\\\\");
                break;
            case '\r':
                res.append("\\r");
                break;
            case '\n':
                res.append("\\n");
                break;
            case '\t':
                res.append("\\t");
                break;
            case '\f':
                res.append("\\f");
                break;
            default:
                res.append(c);
            }
        }
        return res.toString();
    }

    /**
     * Unescape special characters (CR, LF, Tab, FF) encoded accroding to
     * Java(C) conventions. It decodes backslashed characters simply by
     * interpreting next characted literally. For example, if you unescape <code>This\ris\na\t\\string\"\'\a\b\c\d</code>
     * you will get <code>ThisCRisLFaTAB\string"'abcd</code>
     *
     * @see Util#escape(String) for escaping strings
     */
    public static String unescape(final String str) {
        final StringBuilder res = new StringBuilder(str.length());
        boolean slashFound = false;
        for (int i = 0; i < str.length(); i++) {
            final char c = str.charAt(i);
            if (slashFound) {
                switch (c) {
                case 'n':
                    res.append('\n');
                    break;
                case 'r':
                    res.append('\r');
                    break;
                case 't':
                    res.append('\t');
                    break;
                case 'f':
                    res.append('\f');
                    break;
                default:
                    res.append(c);
                    break;
                }
                slashFound = false;
            } else if (c == '\\') {
                slashFound = true;
            } else
                res.append(c);
        }
        return res.toString();
    }

    /**
     * Encode string as if we wanted to use it in source code (means escape it
     * and put quotes around it). So, for ex. if you encode <code>ThisCRisLFaTAB\string"'</code>
     * You will get <code>"This\ris\na\t\\string\"\'"</code>
     *
     * @see Util#unstringalize(String) for converting back and
     *      Util#escape(String) for escaping strings without putting them into
     *      quotes.
     */
    public static String stringalize(final String str) {
        if (str == null)
            return "null";
        else
            return '\"' + escape(str) + '\"';
    }

    /**
     * Decode string in source code format by unescaping it and removing quotes
     * around it. Recognizes single quotes and double quotes (Java recognizes
     * only double quotes). For ex. if you decode <code>"This\ris\na\t\\string\"\'\a\b\c\d"</code> You will get <code>ThisCRisLFaTab\string"'abcd</code>
     *
     * @throws StringFormatException
     *             when str is not enclosed in quotes properly
     */
    public static String unstringalize(final String str)
            throws StringFormatException {
        if (str.length() < 2)
            throw new StringFormatException(str, "String too small");
        if (str.charAt(0) != '\"')
            throw new StringFormatException(str, "Missing left parentize");
        if (str.charAt(str.length() - 1) != '\"')
            throw new StringFormatException(str, "Missing right parentize");
        return unescape(str.substring(1, str.length() - 1));
    }

    /**
     * Encode string into apllication/www-urlencoded format
     *
     * See {@link Util#urlencode(String)} for decoding strings
     */
    public static String urlencode(final String val) {
        try {
            return URLEncoder.encode(val, "UTF-8");
        } catch (final UnsupportedEncodingException e) {
            throw new FatalError(e);
        }
    }

    /**
     * Encode string into apllication/www-urlencoded format. Same as {@link Util#urlencode(String)}, but encodes spaces
     * as %20, not as +
     *
     * See {@link Util#urlencode(String)} for decoding strings
     */
    public static String urlencode2(final String val) {
        try {
            return URLEncoder.encode(val, "UTF-8").replaceAll("\\+", "%20");
        } catch (final UnsupportedEncodingException e) {
            throw new FatalError(e);
        }
    }
    
    /**
     * Decode string from apllication/www-urlencoded format
     *
     * See {@link Util#urldecode(String)} for encoding strings
     */
    public static String urldecode(final String encoded) {
    	return urldecode(encoded, "UTF-8");
    }

    public static String urldecode(final String encoded, final String charset) {
        try {
            return URLDecoder.decode(encoded, charset);
        } catch (final UnsupportedEncodingException e) {
            throw new FatalError(e);
        }
    }

   /**
     * Parse parameters in application/www-urlencoded format. Note that you
     * should never pass whole url to this function. You should pass parameters
     * part (that comes after '?') only, so <code>
     * urlParamsParse("http://www.my.com/cgi?param=v1&param2=v2"); // Wrong!
     * urlParamsParse("param=v1&param2=v2"); //Right
     * </code>
     *
     * Since there can be multiple values for each parameter, this funciton
     * returns List&lt;String&gt; for every parameter. To simplify this, please
     * consider {@link GetParameters} class
     */
    public static Map<String, List<String>> urlParamsParse(
    		final String source, final String charset
    ){
        assert !source.startsWith("http://") :
        	"You should pass parameters part (after '?') of the query !";
        final Map<String, List<String>> res =
        	new HashMap<String, List<String>>();
        if (source.length()==0) return res;
        for (String part : StringUtil.split(source, '&')) {
            // Split name/val
            final String[] nameval = StringUtil.split(part, '=', 1);
            final String name = Util.urldecode(nameval[0], charset),
            	         value;
            if (nameval.length == 1)
                value = "";
            else
            	value = Util.urldecode(nameval[1], charset);

            // Get name bag or create new
            final List<String> bag;
            if (res.containsKey(name))
                bag = res.get(name);
            else {
                bag = new ArrayList<String>();
                res.put(name, bag);
            }

            // Add value to bag
            bag.add(value);
        }
        return res;
    }

	public static<E> int indexOf(final Iterable<WeakReference<E>> coll, final E object) {
		if (object == null) return -1;
		final Iterator<WeakReference<E>> it = coll.iterator();
		int i=0;
		while(it.hasNext()) {
			final E obj = it.next().get();
			if (obj != null && object.equals(obj)) return i;
			i++;
		}
		return -1;
	}

	/**
	 * Encode uri path part into application/www-urlencoded format, not encoding slashes or dynamic url part 
	 * (part after '?'). See {@link Util#urlencode(String)}.
	 * For example, the uri
	 * "www.auto.ru/%D0%B0%D0%B2%D1%82%D0%BE/%D0%BC%D0%B0%D1%88%D0%B8%D0%BD%D0%B0/?q=%D0%B0%D0%B2%D1%82%D0%BE"
	 * would be decoded to
	 * "www.auto.ru/авто/машина/?q=%D0%B0%D0%B2%D1%82%D0%BE"
	 */
	public static String urlencodePath(final String uri) {
		int end = uri.indexOf('?');
		if (end<0) end = uri.length();
		final int estimation = end<<2; // each character can potentially be expanded to 1-5 characters.
		                               // 4 is selected as average-case
		final StringBuilder res = new StringBuilder(estimation);
		int pos = 0;
		int next = uri.indexOf('/', 0);
		while (next>=0 && next<end) {
			if (next>pos) res.append(urlencode2(uri.substring(pos, next)));
			res.append('/');
			pos = next+1;
			next = uri.indexOf('/', pos);
		}
		if (pos<end) res.append(urlencode2(uri.substring(pos, end)));
		if (pos<uri.length()) res.append(uri.substring(end));
		return res.toString();
	}

	/**
	 * Decode uri path part from application/www-urlencoded format, not decoding slashes or dynamic url part 
	 * (part after '?'). See {@link Util#urldecode(String)}.
	 * For example, the uri
	 * "www.auto.ru/авто/машина/?q=авто"
	 * would be encoded as
	 * "www.auto.ru/%D0%B0%D0%B2%D1%82%D0%BE/%D0%BC%D0%B0%D1%88%D0%B8%D0%BD%D0%B0/?q=авто"
	 */
	public static String urldecodePath(final String uri) {
		final int end = uri.indexOf('?');
		if (end >= 0)
			return urldecode(uri.substring(0, end))+uri.substring(end); 
		else
			return urldecode(uri);
	}

	final static Random random = new Random();
	
	/**
	 * Selects random elements from list with no duplicates. The order is guaranteed to be same as in source list.
	 * This overload uses standard java.util.Random generator. 
	 * @see Util#randomSample(Random, Collection, Object[])
	 * 
	 * @param source from where elements should be selected
	 * @param dest where random elements should be stored. dest.length should be less or equal to source.size()
	 * @throws IllegalArgumentException when dest.length &gt;= source.size()
	 * @return random subset 
	 */
	public static<E> void randomSample(final Collection<E> source, final E [] dest) 
		throws IllegalArgumentException 
	{
		randomSample(random, source, dest);
	}
	
	/**
	 * Selects random elements from list with no duplicates. The order is guaranteed to be same as in source list.
	 * @param source from where elements should be selected
	 * @param dest where random elements should be stored. dest.length should be less or equal to source.size()
	 * @param rnd random generator to use for sampling 
	 * @throws IllegalArgumentException when dest.length &gt;= source.size()
	 * @return random subset 
	 */
	public static<E> void randomSample(final Random rnd, final Collection<E> source, final E [] dest) 
		throws IllegalArgumentException 
	{
		if (dest.length==0) return;
		final int size = source.size();
		if (dest.length==size) {
			 source.toArray(dest);
			 return;
		} 
		if (dest.length>size) {
			throw new IllegalArgumentException("dest.length="+dest.length+">source.size="+size);
		}
		
		final Iterator<E> it = source.iterator();
		int pos = 0;
		int left = size;
		for (;;) {
			final E current = it.next(); 

			final int missing = dest.length - pos;
			final int roll = rnd.nextInt(left); //[0..left-1]
			// random chance is = missing/left
			final boolean currentElementSelected = roll < missing;
			if (currentElementSelected) {
				dest[pos++] = current;
				if (missing == 1) break;
			}
			
			left--;
		}
	}
	
	/**
	 * Return copy of the collection as list
	 */
	public static<E> List<E> list(final Collection<E> coll) {
		final List<E> res = new ArrayList<E>(coll.size());
		
		for (final E e : coll) {
			res.add(e); 
		}

		return res;
	}

	public static MessageDigest getMD5Digest() {
		try {
			return MessageDigest.getInstance("MD5");
		} catch (final NoSuchAlgorithmException e) {
			throw new FatalError(e);
		}
	}

	/**
	 * Calculate MD5 over the UTF-8 string and return as string. For example, 
	 * Util.MD5("AAA\n") will return "8880cd8c1fb402585779766f681b868b",
	 * Util.MD5("") will return "d41d8cd98f00b204e9800998ecf8427e",
	 */
	public static String MD5(final String string) {
		return MD5(string, "UTF-8");
	}
	
	/**
	 * Calculate MD5 over the string and return as string. For example, 
	 * Util.MD5("AAA\n", "ASCII") will return "8880cd8c1fb402585779766f681b868b",
	 * Util.MD5("", "UTF-8") will return "d41d8cd98f00b204e9800998ecf8427e",
	 */
	public static String MD5(final String string, final String charset) {
		
		final MessageDigest md = getMD5Digest();
		
		final byte [] data;
		try {
			data = string.getBytes(charset);
		} catch (final UnsupportedEncodingException e) {
			throw new FatalError(e);
		}
		
		md.update(data);
		
		return hexString(md.digest());
	}

	/**
	 * For example: Util.hexString(new byte[]{(byte)0xca, (byte)0xfe}) will return "cafe"
	 * @return hex string representing the byte array
	 */
	public static String hexString(final byte[] bs) {
		final StringBuilder res = new StringBuilder(bs.length<<1);
		for (final byte b : bs) {
			int i = b;
			if (i<0) i+=256;
			if (i<0x10) res.append('0');
			res.append(Integer.toHexString(i));
		}
		return res.toString();
	}

}
