/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.util.fs.transactions;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import junit.framework.Assert;

import org.builtu.testing.BasicTestCase;
import org.builtu.util.fs.transactions.FileTransaction;
import org.builtu.util.fs.transactions.TransactionFixException;
import org.junit.Before;
import org.junit.Test;


public class TestFileTransaction extends BasicTestCase {
	
	private File f;
	
	@Before public void makeFile() throws IOException {
		f = super.createTempFile();
		final OutputStream out = new FileOutputStream(f);
		try {
			for (int line=0; line<100; line++) {
				final byte [] data = ("line"+line+"\n").getBytes("ASCII");
				out.write(data);
			}
		} finally {
			out.close();
		}
	}
	
	@Test public void test() throws IOException, TransactionFixException {
		final FileTransaction t1 = new FileTransaction(f.getPath(), "r");
		try {
			Assert.assertEquals("line0", t1.getFile().readLine());
			Assert.assertEquals("line1", t1.getFile().readLine());
			Assert.assertEquals("line2", t1.getFile().readLine());
			t1.commit();
			Assert.assertEquals("line3", t1.getFile().readLine());
		} finally {
			t1.close();
		}
		
		final FileTransaction t2 = new FileTransaction(f.getPath(), "r");
		try {
			Assert.assertEquals("line3", t2.getFile().readLine());
			Assert.assertEquals("line4", t2.getFile().readLine());
		} finally {
			t1.close();
		}
	}
}
