package org.builtu;

import java.util.ArrayList;
import java.util.List;

import junit.framework.Assert;

import org.junit.Test;

public class TestAggregatorThreaded extends TestAggregator {
	
	private List<Thread> threads = new ArrayList<Thread>();
	
	private void keepAliveInThread(final String ... sessions) {
		keepAliveInThread(1, 0, sessions);
	}

	private void keepAliveInThread(final int times, final int sleepTime, final String ... sessions) {
		final Thread t = new Thread(new Runnable() {
			@Override public void run() {
				for (int i=0; i<times; i++) {
					for (final String session : sessions) {
						if (sleepTime>0)
							try {
								Thread.sleep(sleepTime);
							} catch (InterruptedException e) {
								e.printStackTrace();
								return;
							}
						aggregator.keepAlive(session);
					}
				}
				aggregator.commit();
			}
		});
		threads.add(t);
		t.start();
	}
	
	private void waitAll() throws InterruptedException {
		for (final Thread t : threads) t.join();
		threads.clear();
	}
	
	@Test public void testSimple() throws InterruptedException {
		keepAliveInThread("session1", "session2", "session3");
		
		waitAll();
		aggregator.setInterval(0);
		aggregator.keepAlive("session1");
		
		Assert.assertEquals(3, listener.maxCount);
	}
	
	@Test public void testLonger() throws InterruptedException {
		keepAliveInThread(100, 0, "session1");
		keepAliveInThread(100, 0, "session2");
		keepAliveInThread(100, 0, "session3");
		
		waitAll();
		aggregator.setInterval(0);
		aggregator.keepAlive("session1");
		
		Assert.assertEquals(3, listener.maxCount);
	}
	
	@Test public void testMixed() throws InterruptedException {
		keepAliveInThread(100, 0, "session1", "session2", "session3");
		keepAliveInThread(100, 0, "session2", "session3", "session1");
		keepAliveInThread(100, 0, "session3", "session2", "session1");
		
		waitAll();
		aggregator.setInterval(0);
		aggregator.keepAlive("session1");
		
		Assert.assertEquals(3, listener.maxCount);
	}
	
	@Test public void test100Threads() throws InterruptedException {
		for (int i=0; i<100; i++) {
			keepAliveInThread(100, 0, "session1", "session2", "session3");
		}
		
		waitAll();
		aggregator.setInterval(0);
		aggregator.keepAlive("session1");
		
		Assert.assertEquals(3, listener.maxCount);
	}
	
	@Test public void simulate() throws InterruptedException {
		aggregator.setInterval(10);
		for (int sessions=1; sessions<10; sessions++) {
			final String [] s = new String[sessions];
			for (int i=0; i<sessions; i++) s [i] = "session"+i;
			for (int threads = 0; threads<sessions; threads++) {
				keepAliveInThread(100, 1, s);
			}
			waitAll();
			Assert.assertEquals(sessions, listener.maxCount);
		}
	}
	
}
